@@ -49,6 +49,9 @@ class BaseAppInfo(TypedDict):
     terms_of_service_url: NotRequired[str]
     privacy_policy_url: NotRequired[str]
     rpc_origins: NotRequired[List[str]]
+    interactions_endpoint_url: NotRequired[Optional[str]]
+    redirect_uris: NotRequired[List[str]]
+    role_connections_verification_url: NotRequired[Optional[str]]
 
 
 class AppInfo(BaseAppInfo):
@@ -64,16 +67,12 @@ class AppInfo(BaseAppInfo):
     tags: NotRequired[List[str]]
     install_params: NotRequired[InstallParams]
     custom_install_url: NotRequired[str]
-    role_connections_verification_url: NotRequired[str]
 
 
 class PartialAppInfo(BaseAppInfo, total=False):
     hook: bool
     max_participants: int
     approximate_guild_count: int
-    redirect_uris: List[str]
-    interactions_endpoint_url: Optional[str]
-    role_connections_verification_url: Optional[str]
 
 
 class GatewayAppInfo(TypedDict):
